package ea;

import java.util.Collections;
import java.util.Set;

public class EA {

	private final Reproducer reproducer;
	private final FitnessTester tester;
	private final AdultSelector selector;

	private Set<Individual> individuals;
	private int generation;

	public EA(Set<Individual> startIndividuals, Reproducer reproducer, FitnessTester tester, AdultSelector selector) {
		this.individuals = startIndividuals;
		generation = 0;
		this.reproducer = reproducer;
		this.tester = tester;
		this.selector = selector;
		tester.testIndividuals(individuals);
	}

	public void nextGeneration() {
		generation++;
		Set<Individual> children = reproducer.generateChildren(this);
		tester.testIndividuals(children);
		individuals = selector.selectAdults(individuals, children);
		printToLog();
	}

	public Set<Individual> getIndividuals() {
		return individuals;
	}

	public int getGeneration() {
		return generation;
	}

	public double maxFitness() {
		Individual individual = Collections.max(individuals);
		return individual.getFitness();
	}

	public double avgFitness() {
		double avg = 0;
		for (Individual individual : individuals) {
			avg += individual.getFitness();
		}
		avg /= individuals.size();
		return avg;
	}

	public double sdFitness() {
		double avg = avgFitness();
		double sd = 0;
		for (Individual individual : individuals) {
			sd += Math.pow(individual.getFitness() - avg, 2);
		}
		sd /= individuals.size();
		sd = Math.sqrt(sd);
		return sd;
	}

	private void printToLog() {
		System.out.println("Generation " + generation + ": max=" + maxFitness() + " avg=" + avgFitness() + " sd="
				+ sdFitness() + " phenome = " + Collections.max(individuals).phenomeString());
	}
}
