package ea;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Random;
import java.util.Set;

public class RankReproducer implements Reproducer {

	private static final Random RANDOM = new Random();

	private int noChildren;
	private double crossover;
	private double mutation;
	private double maxMinRatio;

	public RankReproducer(int noChildren, double crossover, double mutation, double maxMinRatio) {
		this.noChildren = noChildren;
		this.crossover = crossover;
		this.mutation = mutation;
		this.maxMinRatio = maxMinRatio;
	}

	@Override
	public Set<Individual> generateChildren(EA ea) {
		Set<Individual> adults = ea.getIndividuals();
		Set<Individual> children = new HashSet<Individual>();
		while (children.size() < noChildren) {
			Individual parentA = selectParent(adults);
			Individual parentB = selectParent(adults);
			Individual child = parentA.mateWith(parentB, crossover, mutation);
			children.add(child);
		}
		return children;
	}

	private Individual selectParent(Set<Individual> adults) {
		Set<ParentCandidate> candidates = new HashSet<ParentCandidate>();
		double totalProbability = 0;
		List<Individual> sortedAdults = new ArrayList<Individual>(adults);
		Collections.sort(sortedAdults);
		for (int i = 0; i < sortedAdults.size(); i++) {
			double probability = 1.0 + (maxMinRatio - 1.0) * i / (sortedAdults.size() - 1);
			candidates.add(new ParentCandidate(sortedAdults.get(i), probability));
			totalProbability += probability;
		}
		double random = totalProbability * RANDOM.nextDouble();
		for (ParentCandidate candidate : candidates) {
			random -= candidate.getProbability();
			if (random < 0) {
				return candidate.getIndividual();
			}
		}
		return null;
	}
}
