package ea;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Random;
import java.util.Set;

public class UniformReproducer implements Reproducer {

	private static final Random RANDOM = new Random();

	private int noChildren;
	private double crossover;
	private double mutation;

	public UniformReproducer(int noChildren, double crossover, double mutation) {
		this.noChildren = noChildren;
		this.crossover = crossover;
		this.mutation = mutation;
	}

	@Override
	public Set<Individual> generateChildren(EA ea) {
		Set<Individual> adults = ea.getIndividuals();
		Set<Individual> children = new HashSet<Individual>();
		while (children.size() < noChildren) {
			Individual parentA = selectParent(adults);
			Individual parentB = selectParent(adults);
			Individual child = parentA.mateWith(parentB, crossover, mutation);
			children.add(child);
		}
		return children;
	}

	private Individual selectParent(Set<Individual> adults) {
		List<Individual> candidates = new ArrayList<Individual>(adults);
		return candidates.get(RANDOM.nextInt(candidates.size()));
	}
}
