package ea_gui;

import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;

public class ReproducerPanel extends JPanel {

	private static final long serialVersionUID = 1L;

	public static final int FITNESS_PROPORTIONATE = 0;
	public static final int SIGMA_SCALING = 1;
	public static final int TOURNAMENT = 2;
	public static final int RANK = 3;
	public static final int UNIFORM = 4;

	private int reproducer = FITNESS_PROPORTIONATE;

	private JRadioButton btnFitnessProportionate = new JRadioButton("Fitness Proportionate");
	private JRadioButton btnSigmaScaling = new JRadioButton("Sigma Scaling");
	private JRadioButton btnTournament = new JRadioButton("Tournament");
	private JRadioButton btnRank = new JRadioButton("Rank");
	private JRadioButton btnUniform = new JRadioButton("Uniform");
	private JLabel lblChildren = new JLabel("#Children:");
	private JTextField txtChildren = new JTextField(5);
	private JLabel lblCrossover = new JLabel("Crossover:");
	private JTextField txtCrossover = new JTextField(5);
	private JLabel lblMutation = new JLabel("Mutation:");
	private JTextField txtMutation = new JTextField(5);
	private JLabel lblGroupSize = new JLabel("Group size:");
	private JTextField txtGroupSize = new JTextField(5);
	private JLabel lblRandomProbability = new JLabel("Random probability:");
	private JTextField txtRandomProbability = new JTextField(5);
	private JLabel lblMaxMinRatio = new JLabel("Max/Min ratio:");
	private JTextField txtMaxMinRatio = new JTextField(5);

	public ReproducerPanel() {
		super(new GridBagLayout());
		setBorder(BorderFactory.createLineBorder(Color.BLACK, 2, true));
		GridBagConstraints c = new GridBagConstraints();
		c.insets.right = 10;
		c.gridx = 0;
		add(btnFitnessProportionate, c);
		c.gridx = 1;
		add(btnSigmaScaling, c);
		c.gridx = 2;
		add(btnTournament, c);
		c.gridx = 3;
		add(btnRank, c);
		c.gridx = 4;
		add(btnUniform, c);
		c.insets.right = 5;
		c.gridx = 5;
		add(lblChildren, c);
		c.gridx = 6;
		add(txtChildren, c);
		c.gridx = 7;
		add(lblCrossover, c);
		c.gridx = 8;
		add(txtCrossover, c);
		c.gridx = 9;
		add(lblMutation, c);
		c.gridx = 10;
		add(txtMutation, c);
		c.gridx = 11;
		add(lblGroupSize, c);
		c.gridx = 12;
		add(txtGroupSize, c);
		c.gridx = 13;
		add(lblRandomProbability, c);
		c.gridx = 14;
		add(txtRandomProbability, c);
		c.gridx = 15;
		add(lblMaxMinRatio, c);
		c.gridx = 16;
		add(txtMaxMinRatio, c);
		btnFitnessProportionate.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				reproducer = FITNESS_PROPORTIONATE;
				updateView();
			}
		});
		btnSigmaScaling.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				reproducer = SIGMA_SCALING;
				updateView();
			}
		});
		btnTournament.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				reproducer = TOURNAMENT;
				updateView();
			}
		});
		btnRank.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				reproducer = RANK;
				updateView();
			}
		});
		btnUniform.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				reproducer = UNIFORM;
				updateView();
			}
		});
		updateView();
	}

	private void updateView() {
		btnFitnessProportionate.setSelected(false);
		btnSigmaScaling.setSelected(false);
		btnTournament.setSelected(false);
		btnRank.setSelected(false);
		btnUniform.setSelected(false);
		switch (reproducer) {
		case FITNESS_PROPORTIONATE:
			btnFitnessProportionate.setSelected(true);
			break;
		case SIGMA_SCALING:
			btnSigmaScaling.setSelected(true);
			break;
		case TOURNAMENT:
			btnTournament.setSelected(true);
			break;
		case RANK:
			btnRank.setSelected(true);
			break;
		case UNIFORM:
			btnUniform.setSelected(true);
			break;
		}
		if (reproducer == TOURNAMENT) {
			lblGroupSize.setEnabled(true);
			txtGroupSize.setEnabled(true);
			lblRandomProbability.setEnabled(true);
			txtRandomProbability.setEnabled(true);
		} else {
			lblGroupSize.setEnabled(false);
			txtGroupSize.setEnabled(false);
			lblRandomProbability.setEnabled(false);
			txtRandomProbability.setEnabled(false);
		}
		if (reproducer == RANK) {
			lblMaxMinRatio.setEnabled(true);
			txtMaxMinRatio.setEnabled(true);
		} else {
			lblMaxMinRatio.setEnabled(false);
			txtMaxMinRatio.setEnabled(false);
		}
	}

	public int getReproducer() {
		return reproducer;
	}

	public int getChildren() {
		return Integer.parseInt(txtChildren.getText());
	}

	public double getCrossover() {
		return Double.parseDouble(txtCrossover.getText());
	}

	public double getMutation() {
		return Double.parseDouble(txtMutation.getText());
	}

	public int getGroupSize() {
		return Integer.parseInt(txtGroupSize.getText());
	}

	public double getRandomProbability() {
		return Double.parseDouble(txtRandomProbability.getText());
	}

	public double getMaxMinRatio() {
		return Double.parseDouble(txtMaxMinRatio.getText());
	}
}
